const path = require('path');
const fs = require('fs');
const YAML = require('yaml');
const VueLoaderPlugin = require('vue-loader/lib/plugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');

const data = fs.readFileSync(path.resolve(__dirname, './manifest.yml'), { encoding: 'utf8' });
const manifest = YAML.parse(data);

module.exports = {
  mode: 'production',
  entry: {
    'clone': './src/packages/clone/index.ts',
  },
  output: {
    filename: '[name].js',
    chunkFilename: 'components/[name].js',
    path: path.resolve(__dirname, 'dist'),
    publicPath: process.env.DEBUG ? '/api/plugin/static/' : `/minio/apps/code_front_plugin/production/${manifest.app.version}/client-side/`,
  },
  resolve: {
    extensions: ['.ts', '.js'], // 添加 .ts 扩展名
  },
  optimization: {
    minimize: false,
    // minimize: true,
    splitChunks: {
      chunks: 'all',
      minSize: 20000,
      maxSize: 70000,
      minChunks: 1,
      maxAsyncRequests: 30,
      maxInitialRequests: 30,
      automaticNameDelimiter: '~',
      cacheGroups: {
        defaultVendors: {
          test: /[\\/]node_modules[\\/]/,
          priority: -10,
          reuseExistingChunk: true,
          filename: '[name].vendor.js',
        },
        // commons: {
        //   test: /[\\/]node_modules[\\/]/,
        //   name: 'vendor',
        //   chunks: 'all'
        // },
        default: {
          minChunks: 2,
          priority: -20,
          reuseExistingChunk: true,
        },
      },
    },
  },
  module: {
    rules: [
      {
        test: /\.vue$/,
        loader: 'vue-loader',
      },
      {
        test: /\.css$/,
        use: [
          // 'vue-style-loader',
          MiniCssExtractPlugin.loader,
          'css-loader'
        ]
      },
      {
        test: /\.ts$/,
        loader: 'ts-loader',
      },
    ],
  },
  plugins: [
    new VueLoaderPlugin(),
    new MiniCssExtractPlugin({
      filename: '[name].css',
      chunkFilename: 'components/[name].css',
    }),
    new CopyWebpackPlugin({
      patterns: [],
    }),
    {
      apply: (compiler) => {
        compiler.hooks.done.tap('GenerateStatsPlugin', (stats) => {
          const statsJson = stats.toJson({
            entrypoints: true,
            chunks: false,
            children: false,
            modules: false,
            assets: false,
            chunkGroups: false,
          });
          const newManifest = {
            ...manifest,
            modules: {
              'code:front': manifest.modules['code:front'].map((m) => {
                return {
                  ...m,
                  files: statsJson.entrypoints[m.key].assets.map((a) => a.name),
                };
              }),
            }
          };
          // 构建完成后，更新manifest的依赖关系
          fs.writeFileSync(path.resolve(__dirname, 'manifest.yml'), YAML.stringify(newManifest));
        });
      }
    }
  ],
};
